

#include <t_services.h>
#include "kernel_id.h"
#include "test.h"
#include <cdefBF533.h>


#define PEEPINGTEST             // Define PEEPING test to see the top 4 of FIFO after enabling SPORT RX

#ifdef PEEPINGTEST
#define SLEEPDURATION 2000
#else
#define SLEEPDURATION 10
#endif

/*
 *  メインタスク
 */
void main_task(VP_INT exinf)
{
    unsigned short dummy;

    vmsk_log(LOG_UPTO(LOG_INFO), LOG_UPTO(LOG_EMERG));
    syslog(LOG_NOTICE, "Sample program starts (exinf = %d).", (INT) exinf);

    syscall(serial_ctl_por(TASK_PORTID,
            (IOCTL_CRLF | IOCTL_FCSND | IOCTL_FCRCV)));

    /*
     *  タスクの起動
    act_tsk(TASK1);
    act_tsk(TASK2);
    act_tsk(TASK3);
     */

    *pSPORT0_TCR1 =
            0 << 14 |   // TCKFE,   0:sample at down edge, 1:sample at up edge
            0 << 13 |   // LATFS,   0:early frame sync, 1:late frame sync
            0 << 12 |   // LTFS,    0:Active high TFS, 1:Active low TFS
            0 << 11 |   // DITFS,   0:data dependent TFS generation, 1:data independent TFS generation
            0 << 10 |   // TFSR,    0:TFS is not required every word, 1:TFS is required every word
            0 << 9  |   // ITFS,    0:external TFS, 1:internal TFS
            0 << 4  |   // TLSBIT,  0:MSB first transmission, 1:LSB first transmission
            0 << 2  |   // TDTYPE   0:normal, 1:reserved, 2:u-law, 3:a-law
            0 << 1  |   // ITCLK,   0:external clock generation, 1:internal clock generation
            0 << 0  ;   // TSPEN    0:Tx disable, 1:Tx enable

    *pSPORT0_TCR2 =
            0 << 10 |   // TRFST,   0:left streo ch first, 1:right stereo ch first
            0 << 9  |   // TSFESE,  0:normal frame sync, 1:LR frame clock
            0 << 8  |   // TXSE,    0:secondary ch disable, 1:secondary ch enable
            0 << 0  ;   // SLEN     0-1:not allowed,2-31:Serial word length - 1


    *pSPORT0_RCR1 =
            0 << 14 |   // RCKFE,   0:sample at down edge, 1:sample at up edge
            0 << 13 |   // LARFS,   0:early frame sync, 1:late frame sync
            0 << 12 |   // LRFS,    0:Active high RFS, 1:Active low RFS
            1 << 10 |   // RFSR,    0:RFS is not required every word, 1:RFS is required every word
            0 << 9  |   // IRFS,    0:external RFS, 1:internal RFS
            0 << 4  |   // RLSBIT,  0:MSB first transmission, 1:LSB first transmission
            0 << 2  |   // RDTYPE   0:zero fill, 1:sign extend, 2:u-law, 3:a-law
            0 << 1  |   // IRCLK,   0:external clock generation, 1:internal clock generation
            0 << 0  ;   // RSPEN    0:Rx disable, 1:Rx enable

    *pSPORT0_RCR2 =
            0 << 10 |   // RRFST,   0:left streo ch first, 1:right stereo ch first
            0 << 9  |   // RSFESE,  0:normal frame sync, 1:LR frame clock
            1 << 8  |   // RXSE,    0:secondary ch disable, 1:secondary ch enable
           29 << 0  ;   // SLEN     0-1:not allowed,2-31:Serial word length - 1
    ssync();


    int count = 0, event = 0;


        // RX FIFOが空になるまで読み捨てる
    while( *pSPORT0_STAT & RXNE)
        dummy = *pSPORT0_RX32;
    /*
     *  メインループ
     */
    do {
        long int rx[4];

            // RX受信を開始する
        *pSPORT0_RCR1 |= RSPEN;     // RX SPORT Enable
        ssync();
            // 3mS スリープ
        tslp_tsk(3);

            // FIFOにデータがたまったはずである。
            // FIFOデータを読みだして下位2bitのみ表示する
        rx[0] = *pSPORT0_RX32 & 0x3;    // 下位2bitのみ抽出
        rx[1] = *pSPORT0_RX32 & 0x3;    // 下位2bitのみ抽出
        rx[2] = *pSPORT0_RX32 & 0x3;    // 下位2bitのみ抽出
        rx[3] = *pSPORT0_RX32 & 0x3;    // 下位2bitのみ抽出

#ifndef PEEPINGTEST
        *pSPORT0_RCR1 &= ~RSPEN;     // RX SPORT disable
        if (rx[0] !=0)
        {
            event ++;
#endif
            syslog( LOG_NOTICE, "RX word : %d,%d,%d,%d", rx[0], rx[1], rx[2], rx[3] );
#ifndef PEEPINGTEST
        }
        count++;
        if (!( count % 1000 ))
            syslog( LOG_NOTICE, "trial : %d, event : %d", count, event);
#endif
            // RX受信を終了する。FIFOにはデータがたまっているはずである。
        *pSPORT0_RCR1 &= ~RSPEN;     // RX SPORT disable
        ssync();
            // スリープ
        tslp_tsk(SLEEPDURATION);

            // RX FIFOが空になるまで読み捨てる
        while( *pSPORT0_STAT & RXNE)
            dummy = *pSPORT0_RX32;


    } while (1);

    syslog(LOG_NOTICE, "Sample program ends.");
    kernel_exit();
}
